<?php
namespace Ak\Image\Processor\Tinify;

use Ak\Image\Image;
use Ak\Image\ImageInterface;
use Ak\Image\Processor\Adapter\AdapterInterface;
use Ak\Image\Processor\ResizeMode;
use Tinify\Client as TinifyClient;
use Tinify\Source;
use Tinify\Tinify;

/**
 * Obrađuje slike koristeći Tinify servis.
 *
 * @package Ak\Image\Processor\Tinify
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/image-processor-tinify
 */
final class TinifyAdapter implements AdapterInterface
{
    /** @var TinifyClient */
    private $client;

    /**
     * Konstruktor.
     *
     * @param TinifyClient $client
     */
    public function __construct(TinifyClient $client)
    {
        $this->client = $client;

        Tinify::setKey('not empty');
        Tinify::setClient($client);
    }

    /**
     * @return TinifyClient
     */
    public function getClient(): TinifyClient
    {
        return $this->client;
    }

    /**
     * Da li ovaj adapter može obraditi datu sliku?
     *
     * @param ImageInterface $image
     * @return bool
     */
    public function canProcessImage(ImageInterface $image): bool
    {
        return in_array($image->getMimeType(), ['image/jpeg', 'image/png']);
    }

    /**
     * Optimizira datu sliku.
     *
     * @param ImageInterface $image
     * @return ImageInterface
     *
     * @throws \Tinify\ConnectionException
     */
    public function optimize(ImageInterface $image): ImageInterface
    {
        $source = $this->getSource($image);
        return new Image($source->toBuffer());
    }

    /**
     * Mijenja veličinu slike.
     *
     * @param ImageInterface $image
     * @param int|null $width
     * @param int|null $height
     * @param ResizeMode|null $mode
     * @return ImageInterface
     * @throws \Tinify\ConnectionException
     */
    public function resize(ImageInterface $image, int $width = null, int $height = null, ResizeMode $mode = null): ImageInterface
    {
        $source = $this->getSource($image);

        $resized = $source->resize(array_filter([
            'method' => (string)($mode ?? new ResizeMode()),
            'width' => $width,
            'height' => $height,
        ]));

        return new Image($resized->toBuffer());
    }

    /**
     * @param ImageInterface $image
     * @return Source
     * @throws \Tinify\ConnectionException
     */
    private function getSource(ImageInterface $image): Source
    {
        /** @noinspection PhpVoidFunctionResultUsedInspection */
        /** @var object $response */
        $response = $this->client->request("post", "/shrink", $image->getContents());
        return new Source($response->headers['location']);
    }
}