<?php
namespace AkTest\Image\Processor\Tinify;

use Ak\Image\ImageInterface;
use Ak\Image\Processor\Adapter\AdapterInterface;
use Ak\Image\Processor\Tinify\TinifyAdapter;
use Ak\Test\AbstractTestCase;
use PHPUnit\Framework\MockObject\MockObject;
use Tinify\Client;
use Laminas\ServiceManager\PluginManagerInterface;

final class TinifyAdapterTest extends AbstractTestCase
{
    public $providers = [
        TestConfigProvider::class,
    ];

    private function getAdapter(Client $client = null): AdapterInterface
    {
        $client = $client ?? $this->createMock(Client::class);

        $container = $this->getContainer([
            'dependencies' => [
                'services' => [
                    'tinify_client' => $client,
                ],
            ],
        ]);

        /** @var PluginManagerInterface $adapters */
        $adapters = $container->get('image_processor.adapters');
        return $adapters->get('tinify');
    }

    private function getMockImage(string $type, string $contents = ''): ImageInterface
    {
        /** @var ImageInterface|MockObject $image */
        $image = $this->createMock(ImageInterface::class);

        $image->method('getMimeType')
            ->willReturn('image/' . $type);

        $image->method('getContents')
            ->willReturn($contents);

        return $image;
    }

    public function testIsRegistered()
    {
        $client = $this->createMock(Client::class);
        $container = $this->getContainer([
            'dependencies' => [
                'services' => [
                    'tinify_client' => $client,
                ],
            ],
        ]);

        /** @var PluginManagerInterface $adapters */
        $adapters = $container->get('image_processor.adapters');
        $this->assertTrue($adapters->has('tinify'));

        /** @var TinifyAdapter $adapter */
        $adapter = $adapters->get('tinify');
        $this->assertInstanceOf(TinifyAdapter::class, $adapter);
        $this->assertSame($client, $adapter->getClient());
    }

    public function testCanProcessJpegAndPng()
    {
        $jpeg = $this->getMockImage('jpeg');
        $png = $this->getMockImage('png');

        $adapter = $this->getAdapter();

        $this->assertTrue($adapter->canProcessImage($jpeg));
        $this->assertTrue($adapter->canProcessImage($png));
    }

    public function testCannotProcessGifsAndSuch()
    {
        $gif = $this->getMockImage('gif');

        $adapter = $this->getAdapter();

        $this->assertFalse($adapter->canProcessImage($gif));
    }

    public function testWillOptimizeTheImageViaTheClient()
    {
        /** @var Client|MockObject $client */
        $client = $this->createMock(Client::class);

        $client->expects($this->exactly(2))
            ->method('request')
            ->willReturn(
                (object) ['headers' => ['location' => 'some-location']],
                (object) ['headers' => [], 'body' => 'optimized image data']
            );

        $image = $this->getMockImage('jpeg');
        $adapter = $this->getAdapter($client);

        $optimized = $adapter->optimize($image);

        $this->assertEquals('optimized image data', $optimized->getContents());
    }

    public function testWillResizeTheImageViaTheClient()
    {
        /** @var Client|MockObject $client */
        $client = $this->createMock(Client::class);

        $client->expects($this->exactly(2))
            ->method('request')
            ->willReturn(
                (object) ['headers' => ['location' => 'some-location']],
                (object) ['headers' => [], 'body' => 'resized image data']
            );

        $image = $this->getMockImage('jpeg');
        $adapter = $this->getAdapter($client);

        $resized = $adapter->resize($image);

        $this->assertEquals('resized image data', $resized->getContents());
    }
}
